﻿Imports C1.Win.C1Chart

Public Class Form1
    Private Sub c1Chart1_Load(sender As Object, e As EventArgs) Handles c1Chart1.Load
        c1Chart1.Header.Text = "Move the mouse over the chart trendlines."
        c1Chart1.Header.Visible = True

        AddChartTrendLines(c1Chart1.ChartGroups.Group0.ChartData)
    End Sub

    Private Sub AddChartTrendLines(chartData As ChartData)
        For si As Integer = 0 To chartData.SeriesList.Count - 1
            Dim cds As ChartDataSeries = chartData.SeriesList(si)
            Dim tl As TrendLine = chartData.TrendsList.AddNewTrendLine()
            tl.SeriesIndex = si
            tl.TrendLineType = TrendLineTypeEnum.Polynom
            tl.RegressionOptions.NumTerms = 4

            Dim c As Color = cds.LineStyle.Color
            tl.LineStyle.Color = Color.FromArgb((c.R + 255) \ 2, (c.G + 255) \ 2, (c.B + 255) \ 2)
            tl.LineStyle.Thickness = 3
        Next
    End Sub

    Private lastHighlightedTrendLineIndex As Integer = -1
    Private Sub processMouseOp(sender As Object, e As MouseEventArgs)
        Dim cg As ChartGroup = c1Chart1.ChartGroups.Group0
        Dim brect As New Rectangle(c1Chart1.ChartArea.PlotArea.Location, c1Chart1.ChartArea.PlotArea.Size)
        Dim maxPixelDistance As Integer = 3

        Dim tlIndex As Integer = FindNearestTrendline(cg, brect, e.Location, maxPixelDistance)

        If tlIndex <> lastHighlightedTrendLineIndex Then
            Dim tlc As TrendLinesCollection = cg.ChartData.TrendsList
            Dim last As Integer = lastHighlightedTrendLineIndex
            If tlIndex >= 0 AndAlso tlIndex < tlc.Count Then
                Dim tl As TrendLine = tlc(tlIndex)
                tl.LineStyle.Thickness *= 2
                lastHighlightedTrendLineIndex = tlIndex
            Else
                lastHighlightedTrendLineIndex = -1
            End If

            If last >= 0 AndAlso last <= tlc.Count Then
                Dim tl As TrendLine = tlc(last)
                tl.LineStyle.Thickness /= 2
            End If

            ' Show status label
            If tlIndex < 0 Then
                label1.Text = "Selection: none"
            Else
                label1.Text = "Selection: " + cg.ChartData.TrendsList(tlIndex).SeriesLabel & " TrendLine"

            End If
        End If
    End Sub

    Private Sub c1Chart1_MouseMove(sender As Object, e As MouseEventArgs) Handles c1Chart1.MouseMove
        processMouseOp(sender, e)
    End Sub

    ''' <summary>
    ''' Given the nearest Mouse coordinate, finds the index of closest trendline less than the
    ''' specified number of pixels distant.  An index of -1 indicates no trendline within the
    ''' specified number of pixels.
    ''' </summary>
    ''' <param name="cg">The ChartGroup with the trendlines to consider.</param>
    ''' <param name="brect">The bounding rectangle of the PlotArea.</param>
    ''' <param name="p">Mouse coordinate.</param>
    ''' <param name="maxPixelDistance">Largest distance in pixels to consider</param>
    ''' <returns></returns>
    Private Function FindNearestTrendline(cg As ChartGroup, brect As Rectangle, p As Point, maxPixelDistance As Integer) As Integer
        Dim tlIndex As Integer = -1

        ' if the point value lies within the chart data coordinate region
        ' there could be a trendline near
        If brect.Contains(p) Then
            Dim cd As ChartData = cg.ChartData
            Dim tlc As TrendLinesCollection = cg.ChartData.TrendsList

            ' find the minimum and maximum X pixel values within range
            Dim pixMinX As Integer = If(((p.X - maxPixelDistance) >= brect.Left), (p.X - maxPixelDistance), brect.Left)
            Dim pixMaxX As Integer = If(((p.X + maxPixelDistance) <= brect.Right), (p.X + maxPixelDistance), brect.Right)

            ' minimum square is enough to identify the closest
            Dim maxDistSquared As Single = maxPixelDistance * maxPixelDistance
            Dim minDistSquared As Single = Single.MaxValue

            For ti As Integer = 0 To tlc.Count - 1
                Dim tl As TrendLine = tlc(ti)
                Dim xf As Single = 0.0F, yf As Single = 0.0F

                ' for each x pixel value, determine the trendline y pixel value
                For px As Integer = pixMinX To pixMaxX
                    Dim xv As Double = 0, yv As Double = 0
                    If cg.CoordToDataCoord(px, p.Y, xv, yv) Then
                        ' get the trendline y value for the calculated x value
                        yv = tl.GetY(xv)

                        ' now get the pixel x and y values
                        If cg.DataCoordToCoord(xv, yv, xf, yf) Then
                            ' calculate and compare minimum distance squared values
                            Dim minds As Single = (xf - p.X) * (xf - p.X) + (yf - p.Y) * (yf - p.Y)
                            If minds < maxDistSquared AndAlso minds < minDistSquared Then
                                ' smallest min distance squared value so far and within range.
                                tlIndex = ti
                                minDistSquared = minds
                                If minds = 0.0F Then
                                    Exit For
                                    ' will not get closer than zero
                                End If
                            End If
                        End If
                    End If
                Next
            Next
        End If
        Return tlIndex
    End Function


End Class
